@@ -260,9 +260,7 @@ impl Wizard {
     pub fn current_widget(&mut self) -> Widget {
         unsafe {
             assert!(!self.was_deleted());
-            Widget::from_widget_ptr(
-                Fl_Wizard_value(self.inner) as *mut fltk_sys::widget::Fl_Widget
-            )
+            Widget::from_widget_ptr(Fl_Wizard_value(self.inner) as *mut fltk_sys::widget::Fl_Widget)
         }
     }
 
@@ -349,19 +347,19 @@ impl Pack {
 /**
     Defines a Vertical Grid (custom widget).
     Requires setting the params manually using the `set_params` method, which takes the rows, columns and spacing.
-    Requires explicit calls to add, which is overloaded especially for the layout.
     ```rust,no_run
     use fltk::{prelude::*, *};
     let mut grid = group::VGrid::new(0, 0, 400, 300, "");
     grid.set_params(3, 3, 5);
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    grid.end();
     ```
 */
 #[derive(Debug, Clone)]
@@ -376,7 +374,6 @@ impl VGrid {
     /// Creates a new vertical grid
     pub fn new<T: Into<Option<&'static str>>>(x: i32, y: i32, w: i32, h: i32, label: T) -> VGrid {
         let vpack = Pack::new(x, y, w, h, label);
-        vpack.end();
         VGrid {
             vpack,
             rows: 1,
@@ -403,7 +400,8 @@ impl VGrid {
 
     /// Adds widgets to the grid
     pub fn add<W: WidgetExt>(&mut self, w: &W) {
-        let rem = self.current / self.cols;
+        debug_assert!(self.current + 1 <= self.rows * self.cols);
+        let rem = (self.current - 1) / self.cols;
         if rem < self.rows {
             let hpack = self.vpack.child(rem as i32).unwrap();
             let mut hpack = unsafe { Pack::from_widget_ptr(hpack.as_widget_ptr()) };
@@ -414,6 +412,32 @@ impl VGrid {
             self.current += 1;
         }
     }
+
+    /// End the grid
+    pub fn end(&mut self) {
+        use std::collections::VecDeque;
+        let children = self.vpack.children();
+        self.current = children - self.rows;
+        debug_assert!(self.current <= self.rows * self.cols);
+        let mut v = VecDeque::new();
+        for i in self.rows..children {
+            let c = self.vpack.child(i).unwrap();
+            v.push_back(c);
+        }
+        for i in 0..self.rows {
+            let hpack = self.vpack.child(i as i32).unwrap();
+            let mut hpack = unsafe { Pack::from_widget_ptr(hpack.as_widget_ptr()) };
+            hpack.end();
+            for _j in 0..self.cols {
+                if let Some(w) = v.pop_front() {
+                    self.vpack.remove(&w);
+                    hpack.add(&w);
+                }
+                hpack.auto_layout();
+            }
+        }
+        self.vpack.auto_layout();
+    }
 }
 
 impl Deref for VGrid {
@@ -433,19 +457,19 @@ impl DerefMut for VGrid {
 /**
     Defines a Horizontal Grid (custom widget).
     Requires setting the params manually using the `set_params` method, which takes the rows, columns and spacing.
-    Requires explicit calls to add, which is overloaded especially for the layout.
     ```rust,no_run
     use fltk::{prelude::*, *};
     let mut grid = group::HGrid::new(0, 0, 400, 300, "");
     grid.set_params(3, 3, 5);
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
-    grid.add(&button::Button::default());
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    button::Button::default();
+    grid.end();
     ```
 */
 #[derive(Debug, Clone)]
@@ -461,7 +485,6 @@ impl HGrid {
     pub fn new<T: Into<Option<&'static str>>>(x: i32, y: i32, w: i32, h: i32, label: T) -> HGrid {
         let mut hpack = Pack::new(x, y, w, h, label);
         hpack.set_type(PackType::Horizontal);
-        hpack.end();
         HGrid {
             hpack,
             rows: 1,
@@ -487,7 +510,8 @@ impl HGrid {
 
     /// Adds widgets to the grid
     pub fn add<W: WidgetExt>(&mut self, w: &W) {
-        let rem = self.current / self.rows;
+        debug_assert!(self.current + 1 <= self.rows * self.cols);
+        let rem = (self.current - 1) / self.rows;
         if rem < self.cols {
             let vpack = self.hpack.child(rem as i32).unwrap();
             let mut vpack = unsafe { Pack::from_widget_ptr(vpack.as_widget_ptr()) };
@@ -498,6 +522,32 @@ impl HGrid {
             self.current += 1;
         }
     }
+
+    /// End the grid
+    pub fn end(&mut self) {
+        use std::collections::VecDeque;
+        let children = self.hpack.children();
+        self.current = children - self.cols;
+        debug_assert!(self.current <= self.rows * self.cols);
+        let mut v = VecDeque::new();
+        for i in self.cols..children {
+            let c = self.hpack.child(i).unwrap();
+            v.push_back(c);
+        }
+        for i in 0..self.cols {
+            let vpack = self.hpack.child(i as i32).unwrap();
+            let mut vpack = unsafe { Pack::from_widget_ptr(vpack.as_widget_ptr()) };
+            vpack.end();
+            for _j in 0..self.rows {
+                if let Some(w) = v.pop_front() {
+                    self.hpack.remove(&w);
+                    vpack.add(&w);
+                }
+                vpack.auto_layout();
+            }
+        }
+        self.hpack.auto_layout();
+    }
 }
 
 impl Deref for HGrid {
@@ -515,23 +565,35 @@ impl DerefMut for HGrid {
 }
 
 /// A wrapper around a vertical pack, with `auto_layout`ing using the add method
+#[derive(Debug, Clone)]
 pub struct Column {
     p: Pack,
 }
 
 impl Column {
+    /// Create a new column
+    pub fn new<T: Into<Option<&'static str>>>(x: i32, y: i32, w: i32, h: i32, label: T) -> Column {
+        let p = Pack::new(x, y, w, h, label);
+        Column { p }
+    }
+
     /// Default init a column filling the parent
     pub fn default() -> Self {
         let mut p = Pack::default().size_of_parent().center_of_parent();
         p.set_type(PackType::Vertical);
-        p.end();
         Column { p }
     }
+
     /// Add a widget to the column with automatic layouting
     pub fn add<W: WidgetExt>(&mut self, w: &W) {
         self.p.add(w);
         self.p.auto_layout();
     }
+
+    /// End the column
+    pub fn end(&mut self) {
+        self.p.auto_layout();
+    }
 }
 
 impl Deref for Column {
@@ -549,23 +611,36 @@ impl DerefMut for Column {
 }
 
 /// A wrapper around a Horizontal pack, with `auto_layout`ing using the add method
+#[derive(Debug, Clone)]
 pub struct Row {
     p: Pack,
 }
 
 impl Row {
+    /// Create a new row
+    pub fn new<T: Into<Option<&'static str>>>(x: i32, y: i32, w: i32, h: i32, label: T) -> Row {
+        let mut p = Pack::new(x, y, w, h, label);
+        p.set_type(PackType::Horizontal);
+        Row { p }
+    }
+
     /// Default init a row filling the parent
     pub fn default() -> Self {
         let mut p = Pack::default().size_of_parent().center_of_parent();
         p.set_type(PackType::Horizontal);
-        p.end();
         Row { p }
     }
+
     /// Add a widget to the row with automatic layouting
     pub fn add<W: WidgetExt>(&mut self, w: &W) {
         self.p.add(w);
         self.p.auto_layout();
     }
+
+    /// End the row
+    pub fn end(&mut self) {
+        self.p.auto_layout();
+    }
 }
 
 impl Deref for Row {
