@@ -492,8 +492,17 @@ impl TextBuffer {
 }
 
 unsafe impl Sync for TextBuffer {}
+
 unsafe impl Send for TextBuffer {}
 
+impl PartialEq for TextBuffer {
+    fn eq(&self, other: &Self) -> bool {
+        self.inner == other.inner
+    }
+}
+
+impl Eq for TextBuffer {}
+
 impl Clone for TextBuffer {
     fn clone(&self) -> TextBuffer {
         assert!(!self.inner.is_null());
@@ -553,6 +562,7 @@ pub enum DragType {
 pub struct TextDisplay {
     inner: *mut Fl_Text_Display,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+    is_derived: bool,
 }
 
 /// Creates an editable text display widget
@@ -560,6 +570,7 @@ pub struct TextDisplay {
 pub struct TextEditor {
     inner: *mut Fl_Text_Editor,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+    is_derived: bool,
 }
 
 /// Creates an editable text display widget to handle terminal-like behavior, such as
@@ -573,6 +584,7 @@ pub struct TextEditor {
 pub struct SimpleTerminal {
     inner: *mut Fl_Simple_Terminal,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+    is_derived: bool,
 }
 
 /// Defines the styles used in the `set_highlight_data`, which is used with style buffers
