@@ -84,6 +84,7 @@ pub enum WindowType {
 pub struct SingleWindow {
     inner: *mut Fl_Single_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+    is_derived: bool,
 }
 
 impl SingleWindow {
@@ -217,6 +218,7 @@ impl SingleWindow {
 pub struct DoubleWindow {
     inner: *mut Fl_Double_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+    is_derived: bool,
 }
 
 impl DoubleWindow {
@@ -315,6 +317,32 @@ impl DoubleWindow {
         unsafe { Fl_Double_Window_flush(self.inner) }
     }
 
+    /// Get the window's opacity
+    pub fn opacity(&self) -> f64 {
+        assert!(!self.was_deleted());
+        unsafe { Fl_Double_Window_alpha(self.inner) as f64 / 255.0 }
+    }
+
+    /// Set the window's opacity, 
+    /// Ranges from 0.0 to 1.0, where 1.0 is fully opaque and 0.0 is fully transparent.
+    /// This should be called on a shown window.
+    /// On X11, opacity support depends on the window manager and can be queried:
+    /// ```ignore
+    /// $ xprop -root _NET_SUPPORTED | grep -o _NET_WM_WINDOW_OPACITY
+    /// ```
+    pub fn set_opacity(&mut self, val: f64) {
+        assert!(!self.was_deleted());
+        assert!(self.shown());
+        let val: u8 = if val > 1.0 {
+            255
+        } else if val < 0.0 {
+            0
+        } else {
+            (val * 255.0).round() as u8
+        };
+        unsafe { Fl_Double_Window_set_alpha(self.inner, val) }
+    }
+
     /// Returns the pixels per unit.
     /// Assumes a factor of 2.0 (Retina) (Retina) on MacOS since `[NSWindow backingScaleFactor]`
     /// requires an info.plist with `NSHighResolutionCapable` set to true.
@@ -357,6 +385,7 @@ impl DoubleWindow {
 pub struct MenuWindow {
     inner: *mut Fl_Menu_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+    is_derived: bool,
 }
 
 impl MenuWindow {
@@ -373,6 +402,7 @@ impl MenuWindow {
 pub struct OverlayWindow {
     inner: *mut Fl_Overlay_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+    is_derived: bool,
 }
 
 impl OverlayWindow {
@@ -512,6 +542,7 @@ pub type GlContext = *mut raw::c_void;
 pub struct GlWindow {
     inner: *mut Fl_Gl_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+    is_derived: bool,
 }
 
 #[cfg(feature = "enable-glwindow")]
@@ -660,6 +691,7 @@ impl GlWindow {
 pub struct GlutWindow {
     inner: *mut Fl_Glut_Window,
     tracker: *mut fltk_sys::fl::Fl_Widget_Tracker,
+    is_derived: bool,
 }
 
 #[cfg(feature = "enable-glwindow")]
