@@ -1,7 +1,6 @@
 use fltk::{app, button::Button, frame::Frame, prelude::*, window::Window};
 
-#[test]
-fn app_handle() {
+fn main() {
     let app = app::App::default();
     let mut wind = Window::default()
         .with_size(400, 300)
@@ -25,8 +24,8 @@ fn app_handle() {
     });
 
     frame.handle(|f, ev| {
-        if ev as i32 > 30 {
-            f.set_label(&format!("{:?}", ev));
+        if ev > 30.into() {
+            f.set_label(&format!("{}", ev));
             true
         } else {
             false
